/* girammultivpan.c
 *
 * Giram, a GPLed 3D modeller.
 * Copyright (C) 2002 DindinX <David@dindinx.org>
 *
 * GTK - The GIMP Toolkit
 * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/*
 * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
 * file for a list of people on the GTK+ Team.  See the ChangeLog
 * files for a list of changes.  These files are distributed with
 * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
 */

#include "girammultivpan.h"

static void giram_multivpan_class_init     (GiramMultivpanClass *klass);
static void giram_multivpan_init           (GiramMultivpan      *vpaned);
static void giram_multivpan_size_request   (GtkWidget           *widget,
                                            GtkRequisition      *requisition);
static void giram_multivpan_size_allocate  (GtkWidget           *widget,
                                            GtkAllocation       *allocation);

static gpointer parent_class;

/*************************************************************************
*  giram_multivpan_get_type
**************************************************************************/
GType giram_multivpan_get_type(void)
{
  static GtkType vpaned_type = 0;

  if (!vpaned_type)
    {
      static const GTypeInfo vpaned_info =
      {
        sizeof(GiramMultivpanClass),
        NULL,     /* base_init */
        NULL,     /* base_finalize */
        (GClassInitFunc)giram_multivpan_class_init,
        NULL,     /* class_finalize */
        NULL,     /* class_data */
        sizeof(GiramMultivpan),
        0,        /* n_preallocs */
        (GInstanceInitFunc)giram_multivpan_init,
      };

      vpaned_type = g_type_register_static(GIRAM_TYPE_MULTIPAN, "GiramMultivpan", 
                                           &vpaned_info, 0);
    }

  return vpaned_type;
}

/*************************************************************************
*  giram_multivpan_class_init
**************************************************************************/
static void giram_multivpan_class_init(GiramMultivpanClass *class)
{
  GtkWidgetClass *widget_class;

  parent_class = gtk_type_class(GIRAM_TYPE_MULTIPAN);

  widget_class = (GtkWidgetClass *)class;

  widget_class->size_request = giram_multivpan_size_request;
  widget_class->size_allocate = giram_multivpan_size_allocate;
}

/*************************************************************************
*  giram_multivpan_init
**************************************************************************/
static void giram_multivpan_init(GiramMultivpan *vpaned)
{
  GiramMultipan *paned;

  g_return_if_fail (GIRAM_IS_MULTIPAN(vpaned));

  paned = GIRAM_MULTIPAN(vpaned);

  paned->cursor_type = GDK_SB_V_DOUBLE_ARROW;
  paned->orientation = GTK_ORIENTATION_HORIZONTAL;
}

/*************************************************************************
*  giram_multivpan_new
**************************************************************************/
GtkWidget *giram_multivpan_new(void)
{
  GiramMultivpan *vpaned;

  vpaned = g_object_new(GIRAM_TYPE_MULTIVPAN, NULL);

  return GTK_WIDGET(vpaned);
}

/*************************************************************************
*  giram_multivpan_size_request
**************************************************************************/
static void giram_multivpan_size_request(GtkWidget      *widget,
                                         GtkRequisition *requisition)
{
  GiramMultipan  *paned = GIRAM_MULTIPAN(widget);
  GtkRequisition  child_requisition;

  requisition->width = 0;
  requisition->height = 0;

  if (paned->child1 && GTK_WIDGET_VISIBLE (paned->child1))
  {
    gtk_widget_size_request(paned->child1, &child_requisition);

    requisition->height = child_requisition.height;
    requisition->width = child_requisition.width;
  }

  if (paned->child2 && GTK_WIDGET_VISIBLE (paned->child2))
  {
    gtk_widget_size_request (paned->child2, &child_requisition);

    requisition->width = MAX (requisition->width, child_requisition.width);
    requisition->height += child_requisition.height;
  }

  requisition->height += GTK_CONTAINER(paned)->border_width * 2;
  requisition->width += GTK_CONTAINER(paned)->border_width * 2;
  
  if (paned->child1 && GTK_WIDGET_VISIBLE(paned->child1) &&
      paned->child2 && GTK_WIDGET_VISIBLE(paned->child2))
  {
    gint handle_size;

    gtk_widget_style_get (widget, "handle_size", &handle_size, NULL);
    requisition->height += handle_size;
  }
}

/*************************************************************************
*  giram_multivpan_size_allocate
**************************************************************************/
static void giram_multivpan_size_allocate(GtkWidget     *widget,
                                          GtkAllocation *allocation)
{
  GiramMultipan *paned        = GIRAM_MULTIPAN(widget);
  gint           border_width = GTK_CONTAINER(paned)->border_width;

  widget->allocation = *allocation;

  if (paned->child1 && GTK_WIDGET_VISIBLE(paned->child1) &&
      paned->child2 && GTK_WIDGET_VISIBLE(paned->child2))
  {
    GtkRequisition child1_requisition;
    GtkRequisition child2_requisition;
    GtkAllocation  child1_allocation;
    GtkAllocation  child2_allocation;
    gint           handle_size;

    gtk_widget_style_get(widget, "handle_size", &handle_size, NULL);

    gtk_widget_get_child_requisition(paned->child1, &child1_requisition);
    gtk_widget_get_child_requisition(paned->child2, &child2_requisition);

    giram_multipan_compute_position(paned,
                                    MAX(1, widget->allocation.height
                                           - handle_size
                                           - 2 * border_width),
                                    child1_requisition.height,
                                    child2_requisition.height);

    paned->handle_pos.x = widget->allocation.x + border_width;
    paned->handle_pos.y = widget->allocation.y + paned->child1_size + border_width;
    paned->handle_pos.width = MAX(1, (gint)widget->allocation.width - 2 * border_width);
    paned->handle_pos.height = handle_size;

    if (GTK_WIDGET_REALIZED(widget))
    {
      gdk_window_show(paned->handle);
      gdk_window_move_resize(paned->handle,
                             paned->handle_pos.x,
                             paned->handle_pos.y,
                             paned->handle_pos.width,
                             handle_size);
    }

    child1_allocation.width = child2_allocation.width = MAX(1, (gint)allocation->width - border_width * 2);
    child1_allocation.height = paned->child1_size;
    child1_allocation.x = child2_allocation.x = widget->allocation.x + border_width;
    child1_allocation.y = widget->allocation.y + border_width;

    child2_allocation.y = child1_allocation.y + child1_allocation.height + paned->handle_pos.height;
    child2_allocation.height = MAX (1, widget->allocation.y + widget->allocation.height - child2_allocation.y - border_width);
      
    /* Now allocate the childen, making sure, when resizing not to
     * overlap the windows
     */
    if (GTK_WIDGET_MAPPED(widget) &&
        paned->child1->allocation.height < child1_allocation.height)
    {
      gtk_widget_size_allocate (paned->child2, &child2_allocation);
      gtk_widget_size_allocate (paned->child1, &child1_allocation);
    }
    else
    {
      gtk_widget_size_allocate (paned->child1, &child1_allocation);
      gtk_widget_size_allocate (paned->child2, &child2_allocation);
    }
  }
  else
  {
    GtkAllocation child_allocation;

    if (GTK_WIDGET_REALIZED(widget))
      gdk_window_hide(paned->handle);

    child_allocation.x = widget->allocation.x + border_width;
    child_allocation.y = widget->allocation.y + border_width;
    child_allocation.width = MAX (1, allocation->width - 2 * border_width);
    child_allocation.height = MAX (1, allocation->height - 2 * border_width);
      
    if (paned->child1 && GTK_WIDGET_VISIBLE (paned->child1))
      gtk_widget_size_allocate(paned->child1, &child_allocation);
    else if (paned->child2 && GTK_WIDGET_VISIBLE (paned->child2))
      gtk_widget_size_allocate(paned->child2, &child_allocation);
  }
}
